title: Engineers Don't Solve Problems
url: https://logicmag.io/failure/engineers-dont-solve-problems/
hash_url: 0e85c776ed374f05913bfdc167823b02

<p>On July 15, 1951, like so many times before, Mexico City’s lakes returned with a vengeance. After heavy rains, the sewers overflowed onto the streets, eventually covering half the city in fetid water. Dramatic photos circulated in newspapers, showing men rowing boats across the city’s downtown streets. The flood paralyzed the capital for ten days and was a major embarrassment for the national government. The event was a spectacular technological failure—the Grand Canal, the state-of-the-art drainage canal built a half-century earlier, had proven totally unable to drain the city. It sat idly by as the city’s residents waded to work.</p>

<p>These kinds of violent, spectacular disasters are what the public has come to understand as a technological failure. But most technological failures, especially when dealing with the environment, are decidedly mundane. They often disproportionately affect the poor in ways that are spatially diffuse and take generations to unfold—a kind of “<a href="http://www.hup.harvard.edu/catalog.php?isbn=9780674072343">slow violence</a>,” as the scholar Rob Nixon has memorably argued. Because of these characteristics, these failures remain largely invisible to those in power and difficult for the majority to fully appreciate. This makes it possible for these technologies to look like successes—until the full extent of their failure is revealed in moments of catastrophe.</p>

<p>The story of Mexico City’s battle against flooding offers a telling lesson for us as we face the slow-motion disaster of climate change. The danger today is that we will again fall for the promise of technological fixes peddled by Silicon Valley entrepreneurs that seem to allow us to continue with business as usual. The problem with these solutions is precisely that they so often <em>appear</em> to work, at least for the groups whose voices count—for now.</p>

<p>We have been thinking about environmental engineering wrong. It does not “solve problems” as is popularly believed. It <em>transforms</em> problems, creating new and different challenges that burden other people—and future generations. The challenge we face as a society is to build the structures of popular power to decide collectively which burdens are worth their weight, and how to distribute them justly. These are not choices we should leave to politicians, or even engineers.</p>

<h3 id="draining-a-sinking-city">Draining a Sinking City</h3>

<p>The official reason for Mexico City’s 1951 flood was <a href="http://www.excelsior.com.mx/comunidad/2014/07/13/970607">clogged drains</a>. But engineers knew something else was to blame: the city was sinking, rendering its drainage system a mess. By the 1940s, scientists and engineers like Nabor Carillo had concrete evidence that this sinking was not natural, but anthropogenic. From the turn of the twentieth century, the rapidly growing city had turned to extracting groundwater using mechanical pumps, depleting the water trapped in the soil below. As the clay soils of the former lakebed upon which the city was built dried out, they shrunk irreversibly, leading to a phenomenon known as “land subsidence”—sinking.</p>

<p>The subsidence had a particularly marked effect on the Grand Canal, a marvel of early twentieth-century hydraulic engineering. The canal, completed in 1900, ostensibly fulfilled the centuries-old project of draining the city’s lakes, which were seen as the cause of flooding—and impediments to urban expansion. Mexico City was trapped at the bottom of a closed valley with no natural rivers flowing in or out. Stretching over thirty miles, the Grand Canal was designed to collect rain and sewage from the city center and take it first east, towards Lake Texcoco, and then through the mountains of the north, where it would be used to irrigate the agricultural fields of the Valle de Mezquital. There was just one problem: the city’s subsidence meant that the canal rapidly lost its slope in the decades after its completion.</p>

<p>By the 1950s, the Grand Canal’s ability to drain the city was already vastly diminished. Engineers began to fear that by the 1970s, the first section of the canal (built on soft, clay soil) would slope towards the city center rather than away from it—rendering it useless. Without the Grand Canal during a major rainstorm, water would accumulate in downtown and turn it into a virtual lake.</p>

<p>To prevent such a catastrophic flood of the city center in the future, engineers after 1951 initiated two major changes. First, they began to move groundwater wells out of the city center and towards the urban periphery, particularly the south and eastern fringes where thousands of new internal migrants were arriving daily from the increasingly destitute countryside. Second, they began studying a radical solution to the Grand Canal’s failure: a system of deep drainage tunnels that would be dug in firm soils less susceptible to subsidence.</p>

<p>The tunnels, which they called the Deep Drainage System (<em>Sistema del Drenaje Profundo</em>) would capture water from the center of the city and use gravity to send it under the mountains to the Valle de Mezquital. There, during the wet summers, a raging torrent of rain, shit, and industrial waste would be used to irrigate the crops that fed the city. This would expand a practice that had begun in earnest with the water of the Grand Canal in 1900.</p>

<p>The Deep Drainage System’s initial phase was completed in 1975. It took over a decade of planning and eight years of dangerous construction, which former workers have described to me as simultaneously awe-inspiring and macabre. (Hundreds are said to have died in its construction, though the number cannot be confirmed with written evidence.) The centerpiece of the system is the Central Emitter Tunnel— nearly twenty feet in diameter, over thirty miles long, and nearly 1000 feet deep in places. The concrete tunnel was large enough that then-President Luis Echevarría was able to tour the completed work in a convoy of trucks, with his full entourage and foreign dignitaries in tow. His government inaugurated the project with great fanfare, releasing full-page ads in every major Mexico City newspaper to declare that the war on the capital’s flooding had finally been won.</p>

<p>The following decades brought a gradual expansion of the tunnel system. It now reaches much of the massive city, like an invisible subway network ninety-five miles long that nearly everyone depends on but no one sees. The project has ostensibly been a huge technological success. Nature appears to have been subdued; the city center never again experienced a flood anywhere close to the magnitude of the 1951 inundation. As a result of the flood protection the system offered, the city was able to continue to grow rapidly without worrying about a large-scale disaster.</p>

<h3 id="robbing-the-future">Robbing the Future</h3>

<p>But this flood protection has come at a steep cost, both for those living on the urban periphery and future generations.</p>

<p>With the Deep Drainage System, the city’s groundwater is pumped from the city’s aquifer and mixed with water imported from other watersheds via massive aqueducts, contaminated, and then mixed with the rain that doesn’t evaporate before it is finally ejected from the watershed through the massive tunnel system. An average of four Olympic-size swimming pools of water are expelled <em>per minute</em> through the tunnels. The result is that wastewater that could have been treated and reused in the city—or rainwater that could have been captured in the hillsides and used to replenish the increasingly parched aquifer—is instead sent out a giant tube.</p>

<p>The result is unsurprising, yet largely invisible to downtown power brokers: the city’s water table is rapidly falling, particularly in the southern and eastern periphery where most of the city’s wells are now located. As a result, wells must be constantly relocated or deepened to access a diminishing resource. Over a million poor residents lack adequate water service, receiving water just a few times a week if at all. Women must stay home to wait for water tanker trucks that may never come, or pay enormous sums for bottled water to perform basic household chores.</p>

<p>Yet this daily deprivation, while at times made visible through popular protests, is largely suffered in silence in the desolate housing blocks of marginalized zones like Iztapalapa. This reality seems worlds away from the gleaming towers of the financial and political elite whose swimming pools never run dry. Across the city, the luxury real estate market has exploded, with new towers sprouting from the rubble of the 2017 earthquake like mushrooms of concrete and steel.</p>

<p>To add insult to injury, the falling water table has provoked severe land subsidence, causing many of the same problems in the periphery that the city center had faced in the decades prior. This has left sewer lines—carefully constructed to flow downhill—flipping like see-saws or simply broken. With even modest rains, these sewers overflow onto local streets and double or triple already grueling commute times, especially for the poor who live far from the city center. Even when the waters do not rise high enough to enter their homes, low-income residents run the risk of infection and ruined clothes trudging through the sewage from these shallow floods.</p>

<p>But the subsidence is uneven. How much a given point sinks—and thus how much its sewers are damaged—depends on its particular geology and its proximity to pumps. As a result, most floods today are patchy. A single image—even from a drone—would be unable to capture the extent of these localized floods, which are dispersed primarily across the poor periphery. They are seen as isolated events in the popular imagination, rather than symptoms of a systemic failure. As a result, they do not provoke the same level of generalized social discontent as the more concentrated flooding disaster of 1951 did.</p>

<p>The Deep Drainage System succeeded precisely by failing in the most mundane and invisible way possible. It transformed a catastrophic problem into a creeping one, out of sight of city elites. In trying to prevent the flooding of the city center, it created a patchwork of flooding along the urban periphery. It displaced the costs of the city’s voracious growth onto the margins, far from the centers of power—and onto future generations.</p>

<h3 id="the-politics-of-poop">The Politics of Poop</h3>

<p>Emboldened by the false sense of security offered by the tunnels and other hydraulic engineering works, government leaders over the decades since 1975 have had no qualms pushing for further growth of the metropolitan region, even as the aquifer dwindles. The growth of the metropolis has not only generated more humans dumping waste. It has also led to more buildings and roads, shrinking the green areas that once allowed water to infiltrate into the groundwater aquifer, rather than run off into the drainage system.</p>

<p>Today, the capacity of the Deep Drainage System is no longer sufficient during the rainy season. During heavy storms, engineers find themselves in an uncomfortable predicament: they have to start closing certain floodgate connections to the surface sewers, or else the tunnels will overflow in spectacular ways they cannot control. With these gates closed, water from the surface sewers has nowhere to go except the streets.</p>

<p>The question of which floodgates to close—and hence <em>whose</em> streets (or whose homes and businesses) will be sacrificed is highly political. It is an open secret that engineers simply aren’t allowed to flood the central neighborhoods where the rich and powerful live. So they will generally close the floodgates in poorer peripheral neighborhoods—often where their own families live—leaving residents to wade through fetid wastewater.</p>

<p>But the sheer quantity of floodwater, combined with deteriorating infrastructure, has eroded the engineers’ control. Standing in their rudimentary command center on the tenth floor of the water utility’s headquarters, there are moments when all they can do is look out the window at the brewing storms and pray. Floods are increasingly reaching the once untouchable neighborhoods and critical infrastructures of the city. Just last year, the airport itself was temporarily shut down due to flooding.</p>

<p>Yet these floods pale in comparison to the city water engineers’ worst nightmare: a collapse of the Central Emitter Tunnel of the Deep Drainage System. This is the system’s main artery, but was designed to function during the rainy season only. Yet in the years after 1975, the city grew exponentially—meaning more sewage and runoff— while the Grand Canal lost even more capacity. This situation forced engineers to use the tunnel year-round just to get the sewage out of the sinking city. With no viable route to divert the sewage, critical maintenance work was delayed for years. This led some engineers by the 1990s to worry the tunnel could collapse due to the degradation of the concrete and steel exposed to years of corrosive gases from wastewater. Such a collapse—in the context of a vastly larger urbanized area—could produce a flood that would make 1951 appear mild in comparison.</p>

<p>To forestall this crisis, the government began building a parallel drainage tunnel in 2008. The Eastern Emitter Tunnel (<em>Túnel Emisor Oriente</em>, or TEO in Spanish) has been touted by its builders as the definitive solution to the region’s flooding problem, and as the longest and most complex drainage tunnel in the world. But standing at the bottom of the tunnel’s deepest underground shaft, large enough to fit a thirty-story building into, it’s hard not to feel that Mexico City, in trying to solve its immediate crises, has dug itself into a hole it will find increasingly difficult to climb out of.</p>

<p>Initially, the TEO will certainly reduce the likelihood of catastrophic floods of the kind that left thousands in the poor peripheral municipality of Chalco with noxious waters in their homes in 2000. It will allow engineers to divert water from the Central Emitter Tunnel and ideally prevent a major failure. But, like its predecessor, it will accelerate the draining of the city’s aquifer and soon be overwhelmed by the very growth it makes possible. Without the TEO, the new airport and its associated real estate developments being pushed by Carlos Slim and foreign investors would be unimaginable. But the airport—and the urbanization it will stimulate — are likely to produce so much new runoff and sewage that in a couple of short decades, the TEO itself will be insufficient.</p>

<p>Yet like the Deep Drainage System before it, politicians and business elites will not judge the TEO by its mundane failures, such as the groundwater depletion and subsidence it facilitates. These effects are slow-moving and concentrated on the urban periphery, far from the centers of power. Instead, elites will consider the TEO a success insofar as it prevents the kind of catastrophic flooding that might stall their dreams of a fast-growing Mexico City.</p>

<h3 id="whacking-moles">Whacking Moles</h3>

<p>The story of Mexico City’s flood protection infrastructure has its unique twists and turns. But it also has the outlines of a broader truth: in engineering, the “success” of a technology often has less to do with solving problems than rendering them opaque or distant from our imagination. Like an endless game of whack-a-mole, the problems never truly go away—they come back with a vengeance decades later and miles away in new forms, often made worse by the very infrastructure engineers created.</p>

<p>This tendency is far from unique to Mexico—or giant sewers. Fossil fuel combustion is a clear example. Like the effects of the Deep Drainage System on residents in the periphery of Mexico City, the effects of burning fuels are felt disproportionately by the poor in the periphery of our capitalist world—in places like Bangladesh, where the sea is slowly swallowing much of the country’s land. Yet this effect is, like Mexico City’s subsidence, nearly invisible—especially to a Wall Street banker or our president.</p>

<p>Even with decades of scientific work proving that our technologies have endangered the very survival of our (and countless other) species, our obsession with economic growth at all costs has barely budged. But if we are to listen to Silicon Valley entrepreneurs and their allies in government and academia, we should not worry about changing our collective way of living on the planet: climate change is simply a problem that can be solved with “disruptive” new engineering innovations, from carbon capture and storage to electric cars.</p>

<p>Yet the story of Mexico City’s struggles over water suggest that we should be skeptical of claims that environmental problems are ever neatly solved through technologies like these. I once asked a Tesla executive who came to Stanford to give a talk whether creating cheap and efficient electric cars wouldn’t simply encourage more driving, more cars, and, further down the line, crises related to lithium mining for batteries in places like Bolivia. (The idea that people with Teslas would drive more is an example of what economists refer to as the “rebound effect”: if you make something more efficient—and hence reduce the cost—people will tend to use it more, whether it’s driving electric cars or taking advantage of flood control infrastructure to build houses in a floodplain.) The executive responded by saying that “those are questions for philosophers—next question?”</p>

<p>These are not questions for philosophers. They are questions for all of us—and especially engineers.</p>

<p>But to be able to wrestle with these questions, we need to change the language we use to think about engineering and technology. Saying engineers “solve problems” implies a kind of mathematical tidiness that doesn’t reflect our messy reality. This language suggests that problems just disappear or are neatly contained through technologies. Yet if Mexico City’s floods are any indication, we should instead talk about how engineers <em>transform</em> problems.</p>

<p>This subtle shift in language brings our attention to the fact that any “solution” produces, inevitably, more and different problems—many of which may not be visible in the moment or place it is implemented, or to the particular group of people designing the intervention. This seems to be, at first glance, obvious. We often say that a given tool “creates more problems than it solves.” Yet the idiom is rarely taken to heart—even if, as engineers, we talk about tradeoffs and generate cost-benefit analyses of different “alternative solutions.” Anyone who has ever worked in an engineering firm or the government knows that these are inevitably influenced by our own biases and interests, whether conscious or not. Furthermore, not every effect of an engineering solution can be quantified in dollars and placed into our analysis.</p>

<p>This is not to say that there are not “better” or “worse” engineering interventions, or that new technologies will not be crucial for dealing with environmental problems. Rather, we must create the popular power necessary to democratically deliberate about these new technologies, and the tradeoffs they represent. We should decide together what kinds of problems we can live with, and what problems we cannot. And we shouldn’t let the promise of magical new technologies distract us from the arduous but essential work of organizing to change our economic system.</p>

<p>The notion that engineers simply “solve problems” is alluring, but dangerously imprecise. It allowed Mexico City’s political class to imagine a city that could grow forever, even while sinking and drying out. As long as engineers appeared to “solve” the city’s most immediate crises, the city’s growth continued. It is only in recent years that citizens have begun to question whether that growth is equitable—and worth the social and environmental cost.</p>

<p>Beyond Mexico City, the fantasy that engineers can wave magic wands and make problems go away is the basis of a global economy built on the fossil fuel extraction that has led our society to the precipice of environmental collapse. Engineers—and our faith in them—make it possible to imagine that the crises we create today will be solved tomorrow by future innovators.</p>

<p>Yet, like the soil underneath Mexico City, this dream is beginning to sink. In recent years, major US cities like New York and Houston have found themselves underwater from storms worsened by climate change. The question is whether we will reverse course before we find ourselves, like Mexico City’s engineers, forced to repeat century-old mistakes just to survive a few years longer.</p>
