title: Les limites pour débattre de numérisation
url: https://gauthierroussilhe.com/post/limites-debat.html
hash_url: f442ffc39b7109ceebe8bc2a669589cb

<p>
    S’intéresser aux impacts environnementaux du numérique crée une friction importante avec différents acteurs du secteur numérique et des nouvelles technologies, friction qui rend l’émergence d’un débat assez laborieux. Je n’arrive pas forcément à trouver tous les acteurs, enjeux et croyances qui rendent compliqué la possibilité d’un débat sur le développement du secteur. J’aimerais cependant lister quelques éléments encombrants que j’ai noté au fur et à mesure des années, sans qu’ils soient nouveaux ou hors du commun. 
</p>

<h3>Table des matières</h3>
<a href="#posture">Un problème de posture</a><br/>
<a href="#decrire">Décrire d’autres numériques</a><br/>
<a href="#valorisation">Sur-valorisation de la numérisation</a><br/>
<a href="#phagocytage">Phagocytage des usages</a><br/>
<a href="#donnees">Sur-valorisation des données numérisées</a><br/>
<a href="#exception">L'exception du débat</a><br/>
<br/>

<h3 id="posture">Un problème de posture</h3>
<p>
    Un des premiers éléments qui rend le dialogue compliqué est que certains professionnels du secteur numérique ont tendance à opérer, me semble t-il, plusieurs amalgames vis à vis de leurs connaissances et de leur transposabilité. <b>Le premier amalgame consiste à supposer que des connaissances en informatique implique par défaut une “culture scientifique”</b>. La “culture scientifique” est un terme chapeau complexe qui décrit des méthodes d’expérimentation normées, des revues par les pairs, des incertitudes sur l’état de ses connaissances, une actualisation récurrente des savoirs. De même, chaque discipline dispose de ses propres méthodologies, héritages, lignées de recherche. Faire de la recherche en anthropologie n’implique pas la même culture scientifique que faire des recherches en physique théorique. En somme, une certaine capacité à la logique acquise par l’apprentissage de la programmation ne garantit en rien une certaine culture scientifique.
</p>

<p>
    Dans la même lignée, avoir des connaissances en informatique ne signifie pas que l’on dispose automatiquement de connaissances en “sciences environnementales” du secteur numérique. S’intéresser aux enjeux environnementaux de ce secteur mobilise des méthodes et des normes propres aux sciences environnementales et qui demandent une certaine expérience pour être comprises et utilisées. De même, la perspective environnementale s’intéresse à l’adéquation matérielle des systèmes numériques dans un milieu contraint. Se demander si telle ou telle chose est soutenable implique de définir par rapport à quoi. En l'occurrence, ce "quoi" désigne les capacités des systèmes terrestres à absorder les dégâts liés aux activités humaines et malgré cela à maintenir un état d'équilibre pour un grand ensemble d'espèces. Ce "quoi" est donc une cible mouvante car on est toujours en train de comprendre comment fonctionnent ces systèmes terrestres, de même pour les effets négatifs des activités humaines dans ces milieux. De plus, ce champ de recherche implique d’aller sur le terrain pour décrire le fonctionnement des systèmes numériques, notamment sur leur aspect matériel et écologique (consommation de ressources naturelles, pollutions, etc.). La plupart de ces éléments sont généralement hors de l’apprentissage reçu en informatique. <b>Que cela soit sur les objets d’étude ou sur les milieux de référence, avoir des connaissances en informatique n’absout en rien d'un apprentissage en sciences environnementales</b>.
</p>


<h3 id="decrire">Décrire d’autres numériques</h3>
<p>
    D’un point de vue matériel le projet de numérisation porté en Europe et en Amérique n’est pas tant différent que le projet porté en Asie : augmentation des flux de données, augmentation de la puissance de calcul et de la connectivité (bande passante et latence des réseaux fixes et mobiles). Ce projet suppose que ces augmentations permettront un développement positif et émancipateur des sociétés humaines, l'émergence d'une sorte de monde numérique humaniste. Cette narration est mobilisée par les grands acteurs du numériques (GAFAM, BATU, etc.), différentes entités industrielles et économiques, et les États avec qui ils font commerce. Toutefois, <b>cela serait être idéaliste de croire que la puissance de calcul et la connectivité sont et seront également distribués dans le monde</b>. Des régimes autoritaires et parfois démocratiques ont et auront recours à des restrictions de connectivité, les centres de données sont loin d’être également répartis sur le globe, et des acteurs privés peuvent tout à fait changer les conditions d’accès aux réseaux ou en contraindre l’accès (prix, accès restreint ou privilégié). Sous prétexte d'une distribution égale de la capacité de connexion le déploiement de réseaux satellitaires en basse-orbite pose précisément les bases de ce dernier point.
</p>

<p>
    De même, il serait naïf de croire que le projet de numérisation et sa progression porte en soi un projet d’émancipation. <b>Il y a et aura des écosystèmes numériques de contrôle pour régimes autoritaires et démocratiques, loin de toute perspective émancipatrice. Il y a et aura des écosystèmes numériques “de classe” qui exacerberont les disparités socio-économiques et l’accès aux services publics comme privés. Il y a et aura des écosystèmes numériques communautaires et/ou distribués pour gérer de façon collective l’infrastructure et l’accès à l’information</b>. L’écosystème numérique des années 90 en Europe ne ressemblent en rien à l’écosystème numérique actuel. C’est bien là la difficulté : tout le monde parle du “numérique” alors qu’il y a “des numériques”, il n’y a donc pas qu’un seul projet de société et ils n’ont pas tous la même puissance. Débattre “du numérique” n’a que peu de sens si on n’a pas défini “le numérique” dont on parle. L’écosystème numérique de la recherche en France (Renater, etc.) n’a peu de choses à voir avec l’écosystème d’un GAFAM alors pourquoi les mettre dans le même panier. Il n’y aura pas de débat sérieux sur la place des infrastructures et des services dans nos sociétés tant que l’on aura pas pluralisé le terme de “numérique” quand on décrit les projets de numérisation que l’on soutient ou que l’on oppose.. 
</p>

<h3 id="valorisation">Sur-valorisation de la numérisation</h3>
<p>
     Un premier élément est la sur-valorisation de la numérisation. Les nouveaux systèmes et usages numériques amènent un praticité apparente qui nous font douter de notre capacité à accomplir quoi que ce soit sans numérisation. <b>C’est cette sur-valorisation qui nous fait dire parfois : “je vois pas comment ils faisaient avant”. À mon sens, ce genre d’affirmations témoigne plutôt d’une méconnaissance et d’une opacification des dispositifs antécédents, et d’un changement de référence d’évaluation</b>. Lors du premier confinement on a souvent mis en avant la nécessaire utilité des outils numériques pour continuer à travailler et apprendre à distance. Aurions-nous pu organiser un enseignement à distance sans une numérisation globalisée ? Il est fort à parier que oui mais pas de la même façon. Le CNED fait de l’enseignement à distance depuis 1939, cet organisme s’est d’ailleurs créé pour palier au dysfonctionnement de l’enseignement lié à la seconde guerre mondiale. Ironiquement, le CNED a connu une panne de ses services numériques liée à un sous-dimensionnement de ses serveurs. Dans un autre registre, a t-on besoin d’outils numériques pour découvrir le réchauffement climatique ? Non, dans la lignée d’Eugène Huzar, Svante Arrhenius publie sa théorie de l’effet de serre en 1896 suite à l’observation de la lune à l’infrarouge.
</p>

<p>
     Est-ce que ces exemples m’amèneraient à dire que c’était mieux avant ou sans numérique, pas du tout. Les systèmes numériques existent et nous en héritons, que nous le voulions ou non. Ce que je cherche à montrer ici est que nous pouvons faire et nous avons fait bien des choses sans numérisation et qu’il s’agit de ne pas l’oublier. <b>Le champ de possibles ne se réduit pas sans numérisation, il est juste redirigé</b>. De même, la numérisation d’un usage le modifie en soi. Une classe à distance a peu de choses à voir avec une classe en présentiel. L’une comme l’autre a des avantages et inconvénients qui s’expriment différemment en fonction des contextes d’application. Il s’agit alors de poser un but (la qualité de la pédagogie), un public et ses caractéristiques, des contraintes immédiates, une orientation stratégique, et un champ de réponses possibles afin de définir les différentes façons de faire (non-numérisées et numérisées).
</p>

<h3 id="phagocytage">Phagocytage des usages</h3>
<p>
    Pour aller plus loin sur cette question il me semble que cette sur-valorisation de la numérisation amène à un second phénomène : le phagocytage des usages. La numérisation a cette incroyable capacité à aborder les usages non-numériques jusqu’à rendre leur idée obsolète et donc à les faire disparaitre. Cela n’a pas que le seul fait de la numérisation mais des dynamiques qui la déploie. Par exemple, dans le cadre de la politique de réduction des coûts d’un état néo-libéral, la numérisation peut servir de levier pour faire disparaitre des services publics physiques dans des zones rurales ou péri-urbaines sous prétexte qu’on maintient un usage ou une fonction grâce à son équivalent numérique. C’est porté peu d’attention sur la façon dont la numérisation modifie les usages ou créer une injonction à la capacité “numérique” des citoyens. <b>Pour pousser le raisonnement à l’absurde le phagocytage du numérique on pourrait tout à fait imaginer que le jour où une société est majoritairement équipée de smartphones avec navigation GPS et d’une couverture stable alors la signalétique publique (panneaux, plaques de rues, directions) peut disparaître. Ainsi, toute orientation dans l’espace dépendra d’un ensemble complexe de logiciels, dépendances, de serveurs, de fibres et d’antennes qui sort de la sphère publique et dont la panne aurait des conséquences beaucoup plus importantes qu’aujourd’hui</b>. 
</p>

<p>
    Cet exemple extrême permet de pousser les limites du raisonnement pour poser une question essentielle : pour quelles raisons et quand faut-il empêcher le phagocytage des usages par la numérisation ? <b>Il est aussi important de rappeler que l’État n’est pas un acteur du numérique à part entière (en tant qu’éditeur de logiciels, ou fournisseurs de solutions/infra) et qu’un phénomène de numérisation amène généralement à un transfert partiel de régime de propriété du public vers la privé</b>. La signalétique publique était en partie à la responsabilité d’acteurs publics (collectivités, équipes de maintenance, etc), les systèmes numériques de navigation (apps, logiciels, etc.) tend globalement vers la privatisation des moyens nécessaires à l’orientation dans l’espace (sauf dans le cas d'OpenStreetMaps).  
</p>


<h3 id="donnees">Sur-valorisation des données numérisées</h3>

<p>
    La numérisation semble aussi avoir un charme puissant envers différents acteurs politiques grâce à une idée qui je n’ai relevé que récemment. Largement inspiré par la théorie de l’information de Shannon, <b>les décideurs de la sphère publique ont été influencées par l’idée que plus de données mènent à de meilleures décisions politiques</b>. C’est un faisceau notamment mobilisé pour justifier par exemple le déploiement de la “smart city” : on assume que plus de capteurs et de connectivité augmente le flux de données dont le croisement révèlerait des meilleures façons de “gérer” la ville et ceux qui l’habitent. Pourtant c’est faire peu de cas de la façon dont sont produites les données, les outils et les acteurs qui les produisent, leur interprétation, etc. La gestion de la crise sanitaire montre bien l’impasse d’une interprétation réductrice de Shannon : confronté aux mêmes courbes et indicateurs (Facteur R, etc.) chaque gouvernement a déployé des stratégies largement différentes, propres aux conditions de leur territoire mais aussi liées à l’interprétation faite des données et à la capacité de certains acteurs de défendre leurs intérêts mieux que d’autres. Produire des données ne garantit rien et sûrement pas des décisions politiques cohérentes avec l’intérêt collectif.
</p>

<h3 id="exception">L'exception du débat</h3>
<p>
    Voilà quelques briques pour peut-être mieux comprendre comment se construit le régime d'exception du secteur numérique. Ce régime va bien au-delà des enjeux environnementaux et se construit aussi dans les questions politiques, économiques, sociales et foncières. La force de cet exceptionnalisme aujourd'hui est un élément bloquant majeur à l'émergence de tout débat sur le développement du secteur numérique et sur la possibilité d'autres numériques. <b>Pourtant il faudra bien se rappeler un jour que le secteur numérique et un secteur comme les autres et qu'il ne pourra pas gagner éternellement une grande partie des arbitrages. D'autant plus que les arbitrages gagnés vont majoritairement à la faveur d'acteurs américains, notamment en France. Il s'agit bien là d'une double peine, climatique et géopolitique</b>.
</p>
