title: Exercices (de feuille) de styles
url: https://blog.professeurjoachim.com/billet/2023-01-05-exercices-de-feuille-de-styles
hash_url: c45d25b1d1062fcf10fbf7caaf9e21b1

<p>Ces temps-ci j’ai environ 4 projets persos démarrés (plus 3 à l’état d’ébauche), auxquels j’ajoute un ou deux projets open source auxquels je contribue. C’est pour ça que quand j’ai cinq minutes, je fais des petits trucs sur mon blog, qui n’est pas comptabilisé dans les projets cités plus haut.</p>
<p>L’avantage avec le blog, c’est que la base est là, la baraque est construite, et que je peux juste faire des expériences de style, comme si je repeignais un mur, mais en moins salissant.</p>
<p>Tel qu’il est actuellement, le blog est propulsé par <a href="https://getkirby.com/">Kirby</a>, mon petit chouchou pour la gestion simple de sites qui n’ont pas de gros besoins de fonctionnalités. J’ai un modèle de page d’accueil qui liste les billets du blog, j’ai un modèle de page de billet, et un modèle de page par défaut.</p>
<p>Les styles ont été conçus à partir de zéro. Mes seules règles : que le site soit lisible et accessible, et que je ne m’empèche pas d’utiliser des propriétés CSS modernes.</p>
<p>Parmi les exemples de propriétés modernes, le mode sombre et les liens.</p>
<p>Les couleurs du site sont gérées par le biais de propriétés CSS personnalisées. Une teinte est définie, et déclinée en plusieurs variantes, nommées en fonction du contraste qu’elles ont avec la couleur de fond.</p>
<pre class="hljs"><code data-language="css"><span class="hljs-selector-pseudo">:root</span> {
    <span class="hljs-attribute">--color-base</span>: <span class="hljs-number">120deg</span>;

    <span class="hljs-attribute">--color-action-low-contrast</span>: <span class="hljs-built_in">hsl</span>(var(--color-base), <span class="hljs-number">80%</span>, <span class="hljs-number">80%</span>);
    <span class="hljs-attribute">--color-action-medium-contrast</span>: <span class="hljs-built_in">hsl</span>(var(--color-base), <span class="hljs-number">70%</span>, <span class="hljs-number">40%</span>);
    <span class="hljs-attribute">--color-action-high-contrast</span>: <span class="hljs-built_in">hsl</span>(var(--color-base), <span class="hljs-number">60%</span>, <span class="hljs-number">25%</span>)
}</code></pre>
<p><strong>Le mode sombre</strong> est géré par une media-query spécifique, qui modifie les propriétés des couleurs, sans toucher à la teinte de base.</p>
<pre class="hljs"><code data-language="css"><span class="hljs-keyword">@media</span> screen <span class="hljs-keyword">and</span> (<span class="hljs-attribute">prefers-color-scheme:</span> dark) {
    <span class="hljs-selector-pseudo">:root</span> {
        <span class="hljs-attribute">--color-action-low-contrast</span>: <span class="hljs-built_in">hsl</span>(var(--color-base), <span class="hljs-number">20%</span>, <span class="hljs-number">25%</span>);
        <span class="hljs-attribute">--color-action-medium-contrast</span>: <span class="hljs-built_in">hsl</span>(var(--color-base), <span class="hljs-number">50%</span>, <span class="hljs-number">50%</span>);
        <span class="hljs-attribute">--color-action-high-contrast</span>: <span class="hljs-built_in">hsl</span>(var(--color-base), <span class="hljs-number">100%</span>, <span class="hljs-number">80%</span>);
    }
}</code></pre>
<p>Pour voir les deux versions, j’ai trouvé cette excellente (et minuscule) extension Firefox, qui permet d’alterner entre le mode clair et le mode sombre, quels que soient les réglages du système ou du navigateur : <a href="https://github.com/Cimbali/toggle-dark-mode">https://github.com/Cimbali/toggle-dark-mode</a>.</p>
<p><strong>Les liens</strong> utilisent des propriétés CSS peu connues, mais très pratiques : <code>text-decoration-thickness</code> qui régit l’épaisseur du trait de décoration (ici un soulignement) et <code>text-underline-offset</code> qui cible le trait de soulignement pour le décaler. Ces deux propriétés sont utilisables sur tous les navigateurs depuis 2021, et elles sont animables via la propriété <code>transition</code>. En animant aussi la couleur du trait de soulignement et celle du texte, <a href="#test-de-lien" id="test-de-lien">on obtient ce résultat</a>. Le seul reproche que je peux faire à cette technique, c’est que je ne peux pas ajouter de marge intérieure au trait (qui est plutôt un bloc coloré qui contient le texte) pour le rendre un peu plus large que le texte qu’il contient.</p>
<h2>Un site en jachère…</h2>
<p>Un blog, ça vit, ça meurt… Je le sais, j’en ai entretenu une demi-douzaine en vingt ans (je viens de vérifier la date d’enregistrement de mon premier nom de domaine : ça dit <code>Creation Date: 2003-01-23T17:55:34Z</code>, j’étais au lycée), et la quasi-totalité ne sont plus entretenus, voire même plus sur le web (hé oui, pour les dix ans de mon premier je l’ai détruit, ce qui lui a valu un petit épitaphe sur le carnet d’un ami).</p>
<p>J’avais envie de présenter le manque d’entretien de ce carnet par un biais amusant et facilement compréhensible. Quand on a un jardin, si on le laisse tranquille plusieurs mois, les herbes folles poussent et les feuilles des arbres couvrent le sol. C’est le temps qui passe, c’est la nature.</p>
<p>Sur ce carnet, c’est pareil. Plus le temps s’écoulera entre deux mises à jour, plus les pages verront des plantes et des feuiles mortes. Si elles deviennent trop nombreuses et gênent à la lecture, <a href="#rateau">il est possible de les ratisser</a>.</p>
<h2>…qui évolue un petit peu selon les saisons</h2>
<p>En restant sur un thème de rythme de la nature, et du fait que ce rythme ne s’applique pas au web, j’ai eu envie d’insérer un autre témoin du temps qui passe. Là, il ne sera pas lié à mon entretien de cet espace web mais plutôt au jour de l’année.</p>
<p>Comme je l’ai abordé plus haut, la couleur est définie au format HSL, ou Hue/Saturation/Lightness, soit <a href="https://fr.wikipedia.org/wiki/Teinte_saturation_luminosit%C3%A9">Teinte/Saturation/Luminosité</a> en français. Grâce à la manière dont j’ai séparé la définition de la teinte dans la définition des couleurs, si je change la valeur de la teinte, les valeurs de saturation et de luminosité ne seront pas touchées. </p>
<p>Les valeurs de teinte sont définies en degré, selon leur angle sur le cercle chromatique. Il y a donc 360 couleurs. Et 365 jours dans l’année. Vous voyez où je veux en venir ?</p>
<p>En arrondissant un peu, chaque jour peut avoir une couleur différente selon son numéro dans l’année. Comment l’avoir ? En PHP c’est facile : <code>&lt;?= date('z') ?&gt;</code> nous fournit la valeur, qu’on colle donc dans une définition de style sur l’élément <code>html</code> : </p>
<pre class="hljs"><code data-language="html"><span class="hljs-tag">&lt;<span class="hljs-name">html</span> <span class="hljs-attr">lang</span>=<span class="hljs-string">"fr"</span> <span class="hljs-attr">style</span>=<span class="hljs-string">"--color-base: &lt;?= date('z') ?&gt;deg;"</span>&gt;</span></code></pre>
<p>Mais attendez ! Le cercle chromatique partirait du rouge en janvier, colorerait le printemps en jaune, l’été en vert puis bleu, et l’automne en violet. J’ai ajusté ça en inversant la valeur du jour, et en choissant un point de départ différent. Comme ça on a bleu (hiver), vert (printemps), orange (été) rouge et violet (automne). Ça se tient mieux, à mon avis.</p>
<p>Et comme je pouvais le faire, j’ai ajouté un filtre à l’icône de raccourci, qui dépend lui aussi de la date actuelle, pour décaler la teinte de l’icône et donc adapter sa couleur en fonction de la couleur principale du blog. L’emoji qui précède le titre du site a lui aussi reçu ce traitement lorsqu’on ne passe pas la souris sur le titre.</p>
<h2>Avec des emoji qui dépassent un peu</h2>
<p>En parlant de l’emoji qui précède le titre, là aussi il y a quelques trucs CSS pour que ça fonctionne.</p>
<p>L’emoji est placé par rapport au positionnement du titre, en retrait par rapport à la colonne de contenu. Jusque là tout va bien. Mais initialement, elle dépassait aussi sur format mobile—et était coupée par le bord de l’écran. Il a fallu trouver une solution.</p>
<p>La colonne de contenu est définie sans avoir besoin de <code>media-query</code>. En effet, ma définition se fonde sur la valeur minimale entre 86% de la largeur de la fenêtre, et la largeur maximum de ligne, qui est de 44 hauteurs de caractère à la taille par défaut. Soit, en CSS :</p>
<pre class="hljs"><code data-language="css"><span class="hljs-selector-pseudo">:root</span> {
    <span class="hljs-attribute">--max-body-width</span>: <span class="hljs-number">86vw</span>;
    <span class="hljs-attribute">--max-line-length</span>: <span class="hljs-number">44rem</span>;
}

<span class="hljs-selector-tag">body</span> {
    <span class="hljs-attribute">width</span>: <span class="hljs-built_in">min</span>(var(--max-body-width), <span class="hljs-built_in">var</span>(--max-line-length));
}</code></pre>
<p>Pour rentrer le titre (et son icône) dans la colonne, il faut ajouter une marge :</p>
<ul>
<li>qui ne soit jamais supérieure à la largeur de l’icône (pour ne pas trop décaler le titre vers la droite)</li>
<li>qui ne soit jamais négative (à l’inverse, pour ne pas décaler le titre vers la droite)</li>
<li>qui dépende de la différence entre la largeur maximale de la ligne et la largeur courante de l’écran (cette valeur est négative en écran large et positive en écran mobile)</li>
</ul>
<p>Ce qui donne, en CSS :</p>
<pre class="hljs"><code data-language="css"><span class="hljs-selector-tag">h1</span> {
    <span class="hljs-attribute">--icon-width</span>: <span class="hljs-number">1.25em</span>;
    <span class="hljs-attribute">margin-left</span>:
        <span class="hljs-built_in">min</span>(
            var(--icon-width),
            <span class="hljs-built_in">max</span>(
                <span class="hljs-number">0px</span>, calc(var(--max-line-length) - <span class="hljs-built_in">var</span>(--max-body-width))
            )
        );
}</code></pre>
<p>Et voilà, suite à ça j’ai pas trop de conclusion à apporter à l’article. Si ça se trouve je recommencerai tout le travail de style dans trois semaines, ou dans trois ans. Ça va dépendre de mon avancée sur mes autres projets, je pense. D’ici là, on va essayer de ne pas laisser pousser trop d’herbes folles.</p>
